//    This file is part of JustVideo.

//    JustVideo is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustVideo is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "actions.h"

Actions::Actions(shared_t *share, QMenuBar *menuBar, QWidget *parent) : QObject(parent)
{
    mainGui = parent;
    conf    = share;

    fileMenu      = new QMenu(tr("File"), mainGui);
    editMenu      = new QMenu(tr("Edit"), mainGui);
    helpMenu      = new QMenu(tr("Help"), mainGui);
    recents       = new QMenu(tr("Recent"), mainGui);
    liveSelect    = new QMenu(tr("Live"), mainGui);
    footageSelect = new QMenu(tr("Footage"), mainGui);

    openFolder       = new QAction(tr("Open Folder..."), this);
    openFolderConf   = new QAction(tr("Open Conf..."), this);
    closeFolder      = new QAction(tr("Close Folder"), this);
    exitApp          = new QAction(tr("Exit"), this);
    about            = new QAction(tr("About ") + QString(APP_NAME) + "...", this);
    pref             = new QAction(tr("Preferences"), this);
    clearRecents     = 0;
    recentsSpacer    = 0;

    rebuildRecents();

    fileMenu->addAction(openFolder);
    fileMenu->addAction(openFolderConf);
    fileMenu->addMenu(recents);
    fileMenu->addSeparator();
    fileMenu->addAction(closeFolder);
    fileMenu->addSeparator();
    fileMenu->addAction(exitApp);

    editMenu->addAction(pref);
    helpMenu->addAction(about);

    connect(openFolder,     &QAction::triggered, this, &Actions::openFldDialog);
    connect(openFolderConf, &QAction::triggered, this, &Actions::openFldConfDialog);
    connect(closeFolder,    &QAction::triggered, this, &Actions::closeSession);
    connect(exitApp,        &QAction::triggered, this, &Actions::closeApp);
    connect(about,          &QAction::triggered, this, &Actions::openAboutDialog);
    connect(pref,           &QAction::triggered, this, &Actions::openAppConfDialog);

    menuBar->addMenu(fileMenu);
    menuBar->addMenu(editMenu);

    live    = menuBar->addMenu(liveSelect);
    footage = menuBar->addMenu(footageSelect);

    menuBar->addMenu(helpMenu);

    closeFolder->setEnabled(false);
}

void Actions::rebuildRecents()
{
    if (recentsSpacer) recentsSpacer->deleteLater();
    if (clearRecents)  clearRecents->deleteLater();

    recents->clear();

    recentsSpacer = recents->addSeparator();
    clearRecents  = new QAction(tr("Clear"), this);

    recents->addAction(clearRecents);

    connect(clearRecents, &QAction::triggered, this, &Actions::rmRecents);
}

void Actions::closeSession()
{
    emit contentClose();

    closeFolder->setEnabled(false);
    footageSelect->clear();
    liveSelect->clear();

    resetSharedRes(conf);
}

void Actions::closeApp()
{
    closeFolder->trigger();

    QCoreApplication::instance()->quit();
}

void Actions::openFld(bool preFill)
{
    if (FolderDialog(conf, preFill, mainGui).exec() == QDialog::Accepted)
    {
        if (closeFolder->isEnabled())
        {
            closeFolder->trigger();
        }

        closeFolder->setEnabled(true);

        emit contentOpen();
    }
}

void Actions::openFldDialog()
{
    openFld(false);
}

void Actions::openFldConfDialog()
{
    auto path = QFileDialog::getOpenFileName(mainGui, tr("Select a config file"), QDir::homePath(), tr("Config File (*.conf)"));

    if (!path.isEmpty())
    {
        openConf(path);
    }
}

void Actions::openAppConfDialog()
{
    PrefDialog(conf, mainGui).exec();
}

void Actions::openConf(const QString &path)
{
    if (closeFolder->isEnabled())
    {
        closeFolder->trigger();
    }

    rdConf(path, conf);
    openFld(true);
}

void Actions::openAboutDialog()
{
    QMessageBox box(mainGui);

    box.setWindowTitle(tr("About ") + QString(APP_NAME));
    box.setTextFormat(Qt::RichText);
    box.setText(QString(APP_NAME) + " v" + QString(APP_VERSION));

    QString info;

    info.append(tr("<p>Based on Qt ") + QString(qVersion()) + " " + QSysInfo::buildCpuArchitecture() + "</p><br>");
    info.append("<p>" + QString(APP_NAME) + tr(" is distributed in the hope that it will be useful,<br>"));
    info.append(tr("but WITHOUT ANY WARRANTY; without even the implied warranty of<br>"));
    info.append(tr("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.</p>"));

    box.setInformativeText(info);
    box.exec();
}

void Actions::rmRecents()
{
    delFolderContents(conf->recentsPath);
}
