//    This file is part of JustVideo.

//    JustVideo is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustVideo is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "common.h"

QStringList lsFiles(const QString &path, const QStringList &filters)
{
    QDir dirObj(path);

    dirObj.setFilter(QDir::Files);
    dirObj.setSorting(QDir::Name);

    if (!filters.isEmpty())
    {
        dirObj.setNameFilters(filters);
    }

    return dirObj.entryList();
}

QStringList lsFiles(const QString &path)
{
    return lsFiles(path, QStringList());
}

QStringList lsVidFiles(const QString &path)
{
    QStringList filters;

    filters << "*.mp4"; filters << "*.ts";  filters << "*.mov";  filters << "*.avi";
    filters << "*.wmv"; filters << "*.flv"; filters << "*.webm"; filters << "*.avchd";
    filters << "*.mkv";

    return lsFiles(path, filters);
}

QStringList lsConfFiles(const QString &path)
{
    QStringList filters;

    filters << "*.conf";

    return lsFiles(path, filters);
}

QStringList lsDirsInDir(const QString &path)
{
    QDir dirObj(path);

    dirObj.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
    dirObj.setSorting(QDir::Name);

    return dirObj.entryList();
}

void rdLine(const QString &param, const QString &line, QString *value)
{
    if (line.startsWith(param))
    {
        *value = line.mid(param.size()).trimmed();
    }
}

void rdLine(const QString &param, const QString &line, int *value)
{
    if (line.startsWith(param))
    {
        *value = line.mid(param.size()).trimmed().toInt();
    }
}

void rdLine(const QString &param, const QString &line, bool *value)
{
    if (line.startsWith(param))
    {
        auto val = line.mid(param.size()).trimmed();
        
        *value = (val == "y" || val == "Y");
    }
}

bool delFolderContents(const QString &path)
{
    auto ret   = false;
    auto files = lsFiles(path);
    auto dirs  = lsDirsInDir(path);

    for (auto &&file: files)
    {
        ret = QFile::remove(path + QDir::separator() + file);
    }

    for (auto &&dir: dirs)
    {
        ret = QDir(dir).removeRecursively();
    }

    return ret;
}

void extCorrection(QString &ext)
{
    if (!ext.startsWith("."))
    {
        ext = "." + ext;
    }
}

bool mkPath(const QString &path)
{
    auto ret = true;
    
    if (!QFileInfo::exists(path))
    {
        ret = QDir(path).mkpath(path);
    }
    
    return ret;
}

void resetSharedRes(shared_t *share)
{
    share->camName.clear();
    share->buffPath.clear();
    share->recPath.clear();
    share->playPath.clear();

    share->conf    = share->appConfPath;
    share->retCode = 0;
    share->isJM    = false;
}

bool runCmd(const QStringList &args, QWidget *parent)
{
    auto ret = true;

    if (!args.isEmpty())
    {
        if (QProcess::execute(args[0], args.mid(1)) != 0)
        {
            QMessageBox::critical(parent, QObject::tr("Exec Error"), QObject::tr("Command: ") + args.join(' ') + QObject::tr(" returned non-zero. See Help > Logs for details"));

            ret = false;
        }
    }

    return ret;
}

bool rdConf(const QString &filePath, shared_t *share)
{
    QFile varFile(filePath);

    if (!varFile.open(QFile::ReadOnly))
    {
        share->retCode = ENOENT;

        QTextStream(stderr) << "err: config file - " << filePath << " does not exists or lack read permissions." << Qt::endl;
    }
    else
    {
        resetSharedRes(share);

        share->conf = filePath;

        QString line;

        do
        {
            line = QString::fromUtf8(varFile.readLine());

            if (!line.startsWith("#"))
            {
                rdLine("cam_name = ",           line, &share->camName);
                rdLine("buffer_path = ",        line, &share->buffPath);
                rdLine("rec_path = ",           line, &share->recPath);
                rdLine("is_jm = ",              line, &share->isJM);
                rdLine("play_path = ",          line, &share->playPath);
                rdLine("max_recents = ",        line, &share->maxRecents);
                rdLine("max_views_per_page = ", line, &share->maxViewsPerPage);
                rdLine("winrect_w = ",          line, &share->windowW);
                rdLine("winrect_h = ",          line, &share->windowH);
                rdLine("winrect_x = ",          line, &share->windowX);
                rdLine("winrect_y = ",          line, &share->windowY);
                rdLine("plistrect_w = ",        line, &share->plistW);
                rdLine("plistrect_h = ",        line, &share->plistH);
                rdLine("plistrect_x = ",        line, &share->plistX);
                rdLine("plistrect_y = ",        line, &share->plistY);
            }

        } while(!line.isEmpty());

        if (share->camName.isEmpty())
        {
            share->camName = QFileInfo(share->conf).baseName();
        }

        if (share->buffPath.isEmpty())
        {
            share->buffPath = "/var/buffer/" + share->camName;
        }
        else
        {
            share->buffPath = QDir::cleanPath(share->buffPath);
        }

        if (share->recPath.isEmpty())
        {
            share->recPath = "/var/footage/" + share->camName;
        }
        else
        {
            share->recPath = QDir::cleanPath(share->recPath);
        }

        qInfo() << "--Conf file: " << filePath << " --";
        qInfo() << "cam_name           = " << share->camName;
        qInfo() << "buffer_path        = " << share->buffPath;
        qInfo() << "rec_path           = " << share->recPath;
        qInfo() << "is_jm              = " << share->isJM;
        qInfo() << "play_path          = " << share->playPath;
        qInfo() << "max_recents        = " << share->maxRecents;
        qInfo() << "max_views_per_page = " << share->maxViewsPerPage;
        qInfo() << "winrect_w          = " << share->windowW;
        qInfo() << "winrect_h          = " << share->windowH;
        qInfo() << "winrect_x          = " << share->windowX;
        qInfo() << "winrcet_y          = " << share->windowY;
        qInfo() << "plistrect_w        = " << share->plistW;
        qInfo() << "plistrect_h        = " << share->plistH;
        qInfo() << "plistrect_x        = " << share->plistX;
        qInfo() << "plistrect_y        = " << share->plistY;
        qInfo() << "-----------------------------------";
    }

    return share->retCode == 0;
}

void wrConf(const QString &filePath, shared_t *share)
{
    QFile       file(filePath);
    QString     txt;
    QTextStream ts(&txt);

    if (file.open(QFile::WriteOnly))
    {
        if (filePath == share->appConfPath)
        {
            ts << "max_recents = "        << share->maxRecents      << Qt::endl;
            ts << "max_views_per_page = " << share->maxViewsPerPage << Qt::endl;
            ts << "winrect_w = "          << share->windowW         << Qt::endl;
            ts << "winrect_h = "          << share->windowH         << Qt::endl;
            ts << "winrect_x = "          << share->windowX         << Qt::endl;
            ts << "winrect_y = "          << share->windowY         << Qt::endl;
            ts << "plistrect_w = "        << share->plistW          << Qt::endl;
            ts << "plistrect_h = "        << share->plistH          << Qt::endl;
            ts << "plistrect_x = "        << share->plistX          << Qt::endl;
            ts << "plistrect_y = "        << share->plistY          << Qt::endl;
        }
        else
        {
            if (!share->playPath.isEmpty()) ts << "play_path = " << share->playPath << Qt::endl;

            if (share->isJM) ts << "is_jm = y" << Qt::endl;
            else             ts << "is_jm = n" << Qt::endl;
        }

        file.write(txt.toUtf8());
    }

    file.close();
}

void createOrUpdateRecent(shared_t *share)
{
    if (share->conf == share->appConfPath)
    {
        QCryptographicHash hasher(QCryptographicHash::Md5);

        hasher.addData(share->playPath.toUtf8());

        if (share->isJM)
        {
            hasher.addData("FF");
        }

        wrConf(share->recentsPath + QDir::separator() + hasher.result().toHex() + ".conf", share);
    }
    else
    {
        wrConf(share->conf, share);
    }
}

void buildGrid(QGridLayout *mainLayout, int parentWidth, const QList<QWidget*> &widList, int cellWidth)
{
    auto columns = 0;

    if (widList.size() == 2)
    {
        columns = 1;
    }
    else
    {
        columns = parentWidth / cellWidth;
    }

    for (auto &&wid : widList)
    {
        mainLayout->removeWidget(wid);
    }

    if (columns != 0)
    {
        for (auto row = 0, i = 0; i < widList.size(); ++row)
        {
            for (auto col = 0; (col < columns) && (i < widList.size()); ++col, ++i)
            {
                mainLayout->addWidget(widList[i], row, col);
            }
        }
    }
    else
    {
        qInfo() << "buildGrid() << calculated a zero column grid. This may cause a blank layout.";
    }
}
