//    This file is part of JustVideo.

//    JustVideo is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustVideo is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "img_loader.h"

ImgLoader::ImgLoader(QObject *parent) : QObject(nullptr)
{
    timer       = nullptr;
    player      = nullptr;
    vidRunning  = false;
    seek        = true;

    auto thr = new QThread(nullptr);

    connect(parent, &QObject::destroyed,   this, &ImgLoader::deleteLater);
    connect(this,   &ImgLoader::destroyed, thr,  &QThread::quit);
    connect(this,   &ImgLoader::loop,      this, &ImgLoader::loadNextImg);
    connect(thr,    &QThread::started,     this, &ImgLoader::loadNextImg);
    connect(thr,    &QThread::finished,    thr,  &QThread::deleteLater);

    moveToThread(thr);

    thr->start();
}

void ImgLoader::resetPlayer()
{
    // QAVPLAYER has an issue with rapid play-stop usage. It will seg fault if
    // I attempt to use the same object in rapid play-stop hence this function
    // that will recreate the object on every play-stop. garbage collection
    // issue? QAVStream::stream() returns null.

    if (player != nullptr)
    {
        player->deleteLater();
    }

    seek   = true;
    player = new QAVPlayer(this);

    connect(player, &QAVPlayer::videoFrame, this, &ImgLoader::rdVideoFrame);
    connect(player, &QAVPlayer::stopped,    this, &ImgLoader::playerDone);
    connect(this,   &ImgLoader::nextVid,    this, &ImgLoader::loadNextImgFromVid);
}

void ImgLoader::playerDone(qint64 pos)
{
    Q_UNUSED(pos);

    auto queuePath = QString(IMG_FROM_VID) + player->source();
    auto label     = vidQueue[queuePath];

    if (label == nullptr)
    {
        qDebug() << "nullptr returned reading from video extraction queue. skipping: " << queuePath;
    }
    else
    {
        vidQueue.remove(queuePath);

        QVideoFrame qframe(rawFrame);
        QPixmap     pix;

        auto img = qframe.toImage();

        if (img.isNull())
        {
            qDebug() << "Failed to convert QImage from raw frame: " << queuePath;
        }
        else if (!pix.convertFromImage(img))
        {
            qDebug() << "Failed to convert QPixmap from QImage: " << queuePath;
        }
        else
        {
            label->setPixmap(pix.scaled(250, 180, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
        }
    }

    resetPlayer();

    emit nextVid();
}

void ImgLoader::rdVideoFrame(const QAVVideoFrame &frame)
{
    if (seek)
    {
        player->seek(player->duration() * 0.10); seek = false;
    }
    else
    {
        rawFrame = frame;

        player->stop();
    }
}

void ImgLoader::loadNextImg()
{
    if (player == nullptr)
    {
        resetPlayer();
    }

    if (timer == nullptr)
    {
        timer = new QTimer(this);

        timer->setInterval(3000);
        timer->setSingleShot(true);

        connect(timer, &QTimer::timeout, this, &ImgLoader::loadNextImg);
    }

    timer->start();

    mutex.lock();

    if (!imgQueue.isEmpty())
    {
        auto keys    = imgQueue.keys();
        auto imgPath = keys.first();
        auto imgBtn  = imgQueue.take(imgPath);

        qInfo() << "Loading thumbnail: " << imgPath << " Queue size: " << imgQueue.size();

        QPixmap pix;

        if (imgPath.startsWith(IMG_FROM_VID))
        {
            vidQueue.insert(imgPath, imgBtn);

            qInfo() << "'" << IMG_FROM_VID << "' tag detected in the thumbnail path, queueing up for capture from video file. Queue size: " << vidQueue.size();
        }
        else if (pix.load(imgPath))
        {
            imgBtn->setPixmap(pix.scaled(250, 180, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
        }
        else
        {
            imgBtn->setPixmap(QPixmap(":/icons/video-250.png"));

            qInfo() << "Failed to load a thumbnail from: " << imgPath << " the format might not be compatible. loading default image.";
        }
    }

    mutex.unlock();

    if (!imgQueue.isEmpty())
    {
        emit loop();
    }
    else if (!vidQueue.isEmpty() && !vidRunning)
    {
        emit nextVid();
    }
}

void ImgLoader::loadNextImgFromVid()
{
    mutex.lock();

    if (!vidQueue.isEmpty())
    {
        vidRunning = true;

        auto keys    = vidQueue.keys();
        auto imgPath = keys.first();
        auto vidPath = imgPath.mid(QString(IMG_FROM_VID).size());

        if (!vidProced.contains(vidPath))
        {
            qDebug() << "Start video file: " << vidPath << " to extract image from video frame.";

            vidProced.append(vidPath);

            player->setSource(vidPath);
            player->play();
        }
    }
    else
    {
        vidRunning = false;
    }

    mutex.unlock();
}

void ImgLoader::addImg(const QString &path, QLabel *label)
{
    mutex.lock();

    imgQueue.insert(path, label);

    mutex.unlock();
}

void ImgLoader::rmImg(const QString &path)
{
    mutex.lock();

    vidQueue.remove(path);
    imgQueue.remove(path);

    mutex.unlock();
}

void ImgLoader::clear()
{
    mutex.lock();

    vidQueue.clear();
    imgQueue.clear();

    mutex.unlock();
}
