//    This file is part of JustVideo.

//    JustVideo is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustVideo is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "player.h"

AudioOut::AudioOut(QObject *parent) : QAVAudioOutput(parent) {}

void AudioOut::playFrame(const QAVAudioFrame &frame)
{
    play(frame);
}

FrameFilter::FrameFilter(QObject *parent) : QObject(parent) {}

Player::Player(QVideoSink *videoSink, bool muted, QWidget *parent) : QAVPlayer(nullptr)
{
    viewWid   = parent;
    vidSink   = videoSink;
    audOutput = nullptr;
    timer     = nullptr;
    curPos    = 0;

    auto thr = new QThread(nullptr);

    if (!muted)
    {
        connect(thr, &QThread::started, this, &Player::setupAudio);
    }

    connect(thr,  &QThread::finished,  this, &Player::deleteLater);
    connect(thr,  &QThread::started,   this, &Player::initTimer);
    connect(this, &Player::videoFrame, this, &Player::setVideoFrame);

    moveToThread(thr);

    thr->start();
}

Player::~Player()
{
    blockSignals(true);
}

void Player::kill()
{
    thread()->quit();
}

void Player::setupAudio()
{
    audOutput = new AudioOut(this);

    connect(this, &Player::audioFrame, audOutput, &AudioOut::playFrame);
}

void Player::initTimer()
{
    timer = new QTimer(this);

    connect(timer, &QTimer::timeout, this, &Player::timeout);

    timer->start(PULL_RATE);
}

QString Player::posDurToTxt(qint64 pos, qint64 dur)
{
    auto posTObj = QTime::fromMSecsSinceStartOfDay(pos);
    auto durTObj = QTime::fromMSecsSinceStartOfDay(dur);

    return posTObj.toString("HH:mm:ss") + "/" + durTObj.toString("HH:mm:ss");
}

QString Player::getSeekPos()
{
    return posDurToTxt(position(), duration());
}

void Player::tooglePlayPause()
{
    if (state() == QAVPlayer::PlayingState)
    {
        pause();
    }
    else
    {
        play();
    }
}

void Player::setVideoFrame(const QAVVideoFrame &frame)
{
    if (viewWid->isVisible() && vidSink != nullptr)
    {
        vidSink->setVideoFrame(frame);
    }
    else
    {
        stop();
    }
}

void Player::timeout()
{
    curPos = position();

    emit seekPos(getSeekPos());
}

void Player::addFrameFilter(FrameFilter *filter)
{
    frameFilters.append(filter);
}

void Player::forward()
{
    seek(curPos + 30000);
}

void Player::back()
{
    seek(curPos - 10000);
}

void Player::setFile(const QString &path)
{
    stop();
    setSource(path);
    play();
}


