//    This file is part of JustVideo.

//    JustVideo is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustVideo is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "playlist_widget.h"

PlaylistItem::PlaylistItem(const QString &filePath, ImgLoader *img, QWidget *parent) : QLabel(parent)
{
    imgLoader  = img;
    vidFile    = filePath;
    popMsg     = new QDialog(this, Qt::Window | Qt::FramelessWindowHint | Qt::Tool);
    popMsgText = new QLabel(popMsg);

    popMsg->hide();
    popMsg->setModal(false);
    popMsg->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

    auto popLayout = new QVBoxLayout(popMsg);

    popLayout->addWidget(popMsgText, 0, Qt::AlignCenter);

    setCursor(Qt::PointingHandCursor);
    setFixedWidth(200);
    setFixedHeight(180);

    genImgAndText();
    setMouseTracking(true);
}

bool PlaylistItem::findImg(QString &pathWithoutExt)
{
    const QStringList imgExts = {".jpg", ".png", ".jpeg", ".bmp", ".gif"};

    for (auto &&ext : imgExts)
    {
        if (QFileInfo::exists(pathWithoutExt + ext))
        {
            pathWithoutExt += ext; return true;
        }
    }

    return false;
}

void PlaylistItem::genImgAndText()
{
    auto info = QFileInfo(vidFile);
    auto name = info.baseName();
    auto dir  = info.path();
    auto imgA = dir + QDir::separator() + name;
    auto imgB = dir + QDir::separator() + ".." + QDir::separator() + "img" + QDir::separator() + name;

    if      (findImg(imgA)) imgFile = imgA;
    else if (findImg(imgB)) imgFile = imgB;
    else                    imgFile = QString(IMG_FROM_VID) + vidFile;

    popMsgText->setText(name);
    imgLoader->addImg(imgFile, this);
}

void PlaylistItem::leaveEvent(QEvent *event)
{
    Q_UNUSED(event);

    popMsg->hide();
}

void PlaylistItem::enterEvent(QEnterEvent *event)
{
    Q_UNUSED(event);

    dispText();
}

void PlaylistItem::mouseReleaseEvent(QMouseEvent *event)
{
    Q_UNUSED(event);

    emit playItemSelected(vidFile);
}

void PlaylistItem::rmVid(const QString &path)
{
    if (path == vidFile)
    {
        emit aboutToDelete(this);

        deleteLater();
    }
}

void PlaylistItem::dispText()
{
    auto pnt = mapToGlobal(QPointF(0, 180));

    popMsg->move(pnt.x(), pnt.y());
    popMsg->show();
}

PlaylistWidget::PlaylistWidget(shared_t *share, PlaylistBE *backend, Player *media, QAction *closeAction, QWidget *parent) : QScrollArea(nullptr)
{
    Q_UNUSED(parent);

    auto listWidget = new QWidget(this);

    player     = media;
    bkend      = backend;
    shared     = share;
    imgLoader  = new ImgLoader(this);
    listLayout = new QGridLayout(listWidget);

    listWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    listWidget->setContentsMargins(0,0,0,0);

    connect(backend,     &PlaylistBE::newFile, this, &PlaylistWidget::addVid);
    connect(closeAction, &QAction::triggered,  this, &PlaylistWidget::closeWid);

    setWindowFlag(Qt::Tool, true);
    setWidget(listWidget);
    setWidgetResizable(true);
    setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
    setFrameStyle(QFrame::NoFrame);

    setGeometry(share->plistX, share->plistY, share->plistW, share->plistH);
}

PlaylistWidget::~PlaylistWidget()
{
    auto curRect = geometry();

    shared->plistX = curRect.x();
    shared->plistY = curRect.y();
    shared->plistW = curRect.width();
    shared->plistH = curRect.height();
}

void PlaylistWidget::closeWid()
{
    imgLoader->blockSignals(true);
    imgLoader->clear();

    deleteLater();
}

void PlaylistWidget::rmVid(const QString &path)
{
    fileList.removeOne(path);
}

void PlaylistWidget::reGrid()
{
    buildGrid(listLayout, width(), widList, 200);
}

void PlaylistWidget::rmItem(const QWidget *item)
{
    widList.removeOne(item); reGrid();
}

void PlaylistWidget::addVid(const QString &path)
{
    if (!fileList.contains(path))
    {
        auto playItem = new PlaylistItem(path, imgLoader, this);

        connect(playItem, &PlaylistItem::playItemSelected, player,   &Player::setFile);
        connect(playItem, &PlaylistItem::aboutToDelete,    this,     &PlaylistWidget::rmItem);
        connect(bkend,    &PlaylistBE::fileDel,            playItem, &PlaylistItem::rmVid);

        fileList.append(path);
        widList.append(playItem);

        reGrid();
    }
}

void PlaylistWidget::showEvent(QShowEvent *event)
{
    reGrid(); QScrollArea::showEvent(event);
}

void PlaylistWidget::resizeEvent(QResizeEvent *event)
{
    reGrid(); QScrollArea::resizeEvent(event);
}

void PlaylistWidget::closeEvent(QCloseEvent *event)
{
    Q_UNUSED(event); hide(); emit closed(false);
}
