//    This file is part of JustVideo.

//    JustVideo is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustVideo is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "vid_widget.h"

VidWidget::VidWidget(shared_t *share, const QString &camName, const QString &playpath, bool multiLive, bool footage, QMenu *camSelect, QAction *closeAction, QStackedWidget *parent) : QVideoWidget(parent)
{
    mainGui     = parent;
    shared      = share;
    live        = multiLive || !footage;
    inGrid      = multiLive;
    menuOption  = nullptr;
    playlistWid = nullptr;
    ctrlOpen    = false;
    objCount    = 2;

    popMsg     = new QDialog(this, Qt::Window | Qt::FramelessWindowHint | Qt::Tool);
    popMsgText = new QLabel(popMsg);
    player     = new Player(videoSink(), multiLive, this);
    playlistBE = new PlaylistBE(share, playpath, player, live, this);

    if (multiLive)
    {
        //setCursor(Qt::PointingHandCursor);
    }
    else
    {
        if (camSelect != nullptr)
        {
            menuOption = camSelect->addAction(camName, this, &VidWidget::viewSelected);
        }

        viewIndex = parent->addWidget(this);

        if (footage)
        {
            playlistWid = new PlaylistWidget(share, playlistBE, player, closeAction, this);

            playlistWid->hide();
        }

        connect(parent, &QStackedWidget::currentChanged, this,   &VidWidget::viewChanged);

        connect(this, &VidWidget::back,       player, &Player::back);
        connect(this, &VidWidget::forward,    player, &Player::forward);
        connect(this, &VidWidget::togglePlay, player, &Player::tooglePlayPause);

        installEventFilter(this);
    }

    popMsg->hide();
    popMsg->setModal(false);
    popMsg->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

    auto popLayout = new QVBoxLayout(popMsg);

    popLayout->addWidget(popMsgText, 0, Qt::AlignCenter);

    connect(this, &VidWidget::start,             playlistBE, &PlaylistBE::start);
    connect(this, &VidWidget::front,             playlistBE, &PlaylistBE::front);
    connect(this, &VidWidget::stop,              player,     &Player::stop);
    connect(this, &VidWidget::fullScreenChanged, this,       &VidWidget::hideMain);

    connect(playlistBE, &PlaylistBE::buffering,       this, &VidWidget::buffering);
    connect(playlistBE, &PlaylistBE::endOfList,       this, &VidWidget::endOfList);
    connect(playlistBE, &PlaylistBE::playbackResumed, this, &VidWidget::resumed);

    connect(closeAction, &QAction::triggered, player,     &Player::kill);
    connect(closeAction, &QAction::triggered, playlistBE, &PlaylistBE::kill);

    connect(player,     &Player::destroyed,     this, &VidWidget::objKilled);
    connect(playlistBE, &PlaylistBE::destroyed, this, &VidWidget::objKilled);

    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    setContentsMargins(0,0,0,0);
}

QPoint VidWidget::getCtrlPopPnt()
{
    return mapToGlobal(QPoint(40, height() - 100));
}

void VidWidget::keyPressEvent(QKeyEvent *event)
{
    if (isVisible() && !inGrid)
    {
        if (event->key() == Qt::Key_Left)
        {
            emit back();
        }
        else if (event->key() == Qt::Key_Right)
        {
            emit forward();
        }
        else if (event->key() == Qt::Key_Space)
        {
            emit togglePlay();
        }
        else if (event->key() == Qt::Key_Escape)
        {
            setFullScreen(false);
        }
    }
}

void VidWidget::popCtrl(const QPointF &pnt)
{
    if (ctrlOpen)
    {
        emit extendControls();
    }
    else
    {
        auto controlsWid = new PlayControl(playlistBE, playlistWid != nullptr, player, this);

        if (playlistWid != nullptr)
        {
            connect(controlsWid, &PlayControl::playlist, playlistWid, &PlaylistWidget::show);
        }

        connect(controlsWid, &PlayControl::accepted, this, &VidWidget::controlsClosed);

        connect(this, &VidWidget::closeControls,  controlsWid, &PlayControl::accept);
        connect(this, &VidWidget::extendControls, controlsWid, &PlayControl::resetHide);

        controlsWid->move(pnt.x(), pnt.y());
        controlsWid->show();

        ctrlOpen = true;
    }
}

bool VidWidget::eventFilter(QObject *obj, QEvent *event)
{
    Q_UNUSED(obj);

    if (isVisible() && !inGrid)
    {
        if (event->type() == QEvent::MouseMove)
        {
            auto pnt     = getCtrlPopPnt();
            auto actRect = QRect(pnt, QSize(300, 100));
            auto curPnt  = QCursor::pos();

            if (actRect.contains(curPnt))
            {
                popCtrl(pnt);
            }
        }
    }

    return false;
}

void VidWidget::controlsClosed()
{
    ctrlOpen = false;
}

void VidWidget::hideMain(bool fsState)
{
    if (fsState)
    {
        emit closeControls();
    }

    emit mainWindowsVis(!fsState);
}

void VidWidget::mouseReleaseEvent(QMouseEvent *event)
{
    Q_UNUSED(event)

    emit togglePlay();
}

void VidWidget::objKilled()
{
    objCount--;

    if (objCount == 0)
    {
        mainGui->removeWidget(this);

        deleteLater();
    }
}

void VidWidget::viewSelected()
{
    mainGui->setCurrentWidget(this);
}

void VidWidget::viewChanged(int index)
{   
    if (viewIndex == index)
    {
        if (live) emit front();
        else      emit start();

        if (menuOption != nullptr)
        {
            menuOption->setIcon(QIcon("://icons/check.png"));
        }
    }
    else
    {
        emit stop();

        if (menuOption != nullptr)
        {
            menuOption->setIcon(QIcon());
        }

        if (playlistWid != nullptr)
        {
            playlistWid->hide();
        }

        popMsg->hide();
    }
}

void VidWidget::dispMsg(const QString &txt)
{
    if (isVisible())
    {
        auto pnt = mapToGlobal(QPointF(10, 10));

        popMsgText->setText(txt);
        popMsg->move(pnt.x(), pnt.y());
        popMsg->show();
    }
}

void VidWidget::resumed()
{
    popMsg->hide();
}

void VidWidget::buffering()
{
    dispMsg(tr("Buffering"));
}

void VidWidget::endOfList()
{
    dispMsg(tr("End-Of-List"));
}

void VidWidget::noMedia()
{
    dispMsg(tr("No-Media"));
}
